<?php

namespace App\Http\Controllers\Admin\Course;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Course\CategoryRepository;
use App\Repositories\Validators\Course\CategoryValidator;

/**
 * Class CourseCategoriesController.
 *
 * @package namespace App\Http\Controllers;
 */
class CategoryController extends Controller
{
    /**
     * @var CategoryRepository
     */
    protected $repository;

    /**
     * @var CategoryValidator
     */
    protected $validator;

    /**
     * CourseCategoriesController constructor.
     *
     * @param CategoryRepository $repository
     * @param CategoryValidator $validator
     */
    public function __construct(CategoryRepository $repository, CategoryValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }


    public function index(Request $request)
    {
        $courseCategories = $this->repository->paginate(request('per_page', self::PAGE_NUM));
        return Response::success($courseCategories);
    }


    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        try {
            $courseCategory = $this->repository->create($request->all());
            return Response::success($courseCategory);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    public function show($id)
    {
        $courseCategory = $this->repository->find($id);
        return Response::success($courseCategory);
    }


    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $courseCategory = $this->repository->update($request->all(), $id);
            return Response::success($courseCategory);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }


    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }


    public function tree()
    {

        $departments = $this->repository->orderByDesc('sort')->get()->append('parent_name')->toArray();

        return Response::success(toTree($departments));
    }

    public function lists(Request $request)
    {
        if ($id = $request->get('id', false)) {
            $category = $this->repository->skipPresenter()->where('status', ModelStatusEnum::OK)->where('parent_id', $id)->get(['id', 'name']);
        } else {
            $category = $this->repository->skipPresenter()->where('status', ModelStatusEnum::OK)->where('parent_id', 0)->get(['id', 'name']);
        }
        return Response::success($category);
    }
}
