<?php

namespace App\Http\Controllers\Admin\Inform;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Inform\Category;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Inform\CategoryRepository;
use App\Repositories\Validators\Inform\CategoryValidator;

/**
 * Class CategoriesController.
 *
 * @package namespace App\Http\Controllers\Admin\Inform;
 */
class CategoryController extends Controller
{
    /**
     * @var CategoryRepository
     */
    protected $repository;

    /**
     * @var CategoryValidator
     */
    protected $validator;

    /**
     * CategoriesController constructor.
     *
     * @param CategoryRepository $repository
     * @param CategoryValidator $validator
     */
    public function __construct(CategoryRepository $repository, CategoryValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    public function index(Request $request)
    {
        $courseCategories = $this->repository->paginate(request('per_page', self::PAGE_NUM));
        return Response::success($courseCategories);
    }


    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $category = Category::query()->create($data);
            return Response::success($category);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    public function show($id)
    {
        $courseCategory = $this->repository->find($id);
        return Response::success($courseCategory);
    }


    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            $category = Category::query()->where('id', $id)->first();
            $category->fill($data);
            $category->save();
            return Response::success($category);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }


    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }


    public function tree()
    {
        $lists = $this->repository->orderByDesc('sort')->get();
        $lists = $this->repository->parserResult($lists)['list'];
        return Response::success(toTree($lists));
    }

    public function lists()
    {
        $user = login_admin();
        if ($user->id == 1) {
            $lists = $this->repository->orderByDesc('sort')->get()->toArray();
        } else {
            $lists = $this->repository->orderByDesc('sort')->where('role_ids', 'like', '%' . $user->role_id . '%')->get()->toArray();
        }

        $lists[] = [
            'id' => 0,
            'name' => '全部',
            'parent_id' => '',
        ];

        return Response::success(toTree($lists));
    }

}
